@@ -9,13 +9,18 @@ import SocketServer
 import httplib
 import urllib
 import urlparse
+import gzip
+import tempfile
+import Cookie
 import rpcrequest
 import rpcresponse
 import rpcerror
 import rpclib
-import Cookie
 from rpcjson import json
 
+MAX_SIZE_IN_MEMORY = 1024 * 1024 * 10  # 10 MiB
+CHUNK_SIZE = 1024 * 1024  # 1 MiB
+
 
 def http_request(
     url,
@@ -25,7 +30,8 @@ def http_request(
     timeout = None,
     additional_headers = None,
     content_type = None,
-    cookies = None
+    cookies = None,
+    gzipped = None
 ):
     """
     Fetch data from webserver (POST request)
@@ -47,11 +53,26 @@ def http_request(
     :param cookies: Possibility to add simple cookie-items as key-value pairs.
         The key and the value of each cookie-item must be a bytestring.
         Unicode is not allowed here.
+
+    :param gzipped: If `True`, the JSON-String will be gzip-compressed.
     """
 
-    request = urllib2.Request(url, data = json_string)
+    # Create request and add data
+    request = urllib2.Request(url)
 
-    # Content-Type
+    if gzipped:
+        # Compress content (SpooledTemporaryFile to reduce memory usage)
+        spooled_file = _SpooledFile()
+        _gzip_str_to_file(json_string, spooled_file)
+        del json_string
+        request.add_header("Content-Encoding", "gzip")
+        request.add_header("Accept-Encoding", "gzip")
+        spooled_file.seek(0)
+        request.add_data(spooled_file)
+    else:
+        request.add_data(json_string)
+
+    # Content Type
     request.add_header("Content-Type", content_type or "application/json")
 
     # Authorization
@@ -69,13 +90,18 @@ def http_request(
         for key, val in additional_headers.items():
             request.add_header(key, val)
 
-    # Request
+    # Send request to server
     response = urllib2.urlopen(request, timeout = timeout)
-    response_string = response.read()
-    response.close()
 
-    # Finished
-    return response_string
+    # Analyze response and return result
+    try:
+        if "gzip" in response.headers.get("Content-Encoding", ""):
+            response_file = _SpooledFile(source_file = response)
+            return _gunzip_file(response_file)
+        else:
+            return response.read()
+    finally:
+        response.close()
 
 
 class HttpClient(object):
@@ -99,7 +125,8 @@ class HttpClient(object):
         timeout = None,
         additional_headers = None,
         content_type = None,
-        cookies = None
+        cookies = None,
+        gzipped = None
     ):
         """
         :param: URL to the JSON-RPC handler on the HTTP-Server.
@@ -121,6 +148,8 @@ class HttpClient(object):
         :param cookies: Possibility to add simple cookie-items as key-value pairs.
             The key and the value of each cookie-item must be a bytestring.
             Unicode is not allowed here.
+
+        :param gzipped: If `True`, the JSON-String will gzip-compressed.
         """
 
         self.url = url
@@ -130,6 +159,7 @@ class HttpClient(object):
         self.additional_headers = additional_headers
         self.content_type = content_type
         self.cookies = cookies
+        self.gzipped = gzipped
 
 
     def call(self, method, *args, **kwargs):
@@ -162,7 +192,8 @@ class HttpClient(object):
             timeout = self.timeout,
             additional_headers = self.additional_headers,
             content_type = self.content_type,
-            cookies = self.cookies
+            cookies = self.cookies,
+            gzipped = self.gzipped
         )
         if not response_json:
             return
@@ -281,6 +312,14 @@ class HttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler, rpclib.JsonRpc):
         self.send_header("Content-Length", str(length))
 
 
+    def set_content_encoding(self, content_encoding):
+        """
+        Set content-encoding to *content_encoding*
+        """
+
+        self.send_header("Content-Encoding", content_encoding)
+
+
     def do_GET(self):
         """
         Handles HTTP-GET-Request
@@ -326,7 +365,7 @@ class HttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler, rpclib.JsonRpc):
                 args = []
                 kwargs = params
 
-        # Create JSON reqeust string
+        # Create JSON request string
         request_dict = rpcrequest.create_request_dict(method, *args, **kwargs)
         request_dict["jsonrpc"] = jsonrpc
         request_dict["id"] = id
@@ -349,9 +388,28 @@ class HttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler, rpclib.JsonRpc):
         Handles HTTP-POST-Request
         """
 
-        # Read JSON request
+        # Read, analyze and parse request
         content_length = int(self.headers.get("Content-Length", 0))
-        request_json = self.rfile.read(content_length)
+        content_encoding = self.headers.get("Content-Encoding", "")
+        accept_encoding = self.headers.get("Accept-Encoding", "")
+
+        if "gzip" in content_encoding:
+            # Decompress
+            with _SpooledFile() as gzipped_file:
+                # ToDo: read chunks
+                # if content_length <= CHUNK_SIZE:
+                #     gzipped_file.write(self.rfile.read(content_length))
+                # else:
+                #     chunks_quantity = content_length % CHUNK_SIZE
+                # ...
+                gzipped_file.write(self.rfile.read(content_length))
+                gzipped_file.seek(0)
+                with gzip.GzipFile(
+                    filename = "", mode = "r", fileobj = gzipped_file
+                ) as gz:
+                    request_json = gz.read()
+        else:
+            request_json = self.rfile.read(content_length)
 
         # Call
         response_json = self.call(request_json) or ""
@@ -360,9 +418,25 @@ class HttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler, rpclib.JsonRpc):
         self.send_response(code = httplib.OK)
         self.set_content_type(self.content_type)
         self.set_no_cache()
-        self.set_content_length(len(response_json))
-        self.end_headers()
-        self.wfile.write(response_json)
+
+        if "gzip" in accept_encoding:
+            # Gzipped
+            content = _SpooledFile()
+            with gzip.GzipFile(filename = "", mode = "w", fileobj = content) as gz:
+                gz.write(response_json)
+            content.seek(0)
+
+            # Send compressed
+            self.set_content_encoding("gzip")
+            self.set_content_length(len(content))
+            self.end_headers()
+            self.wfile.write(content.read())
+        else:
+            # Send uncompressed
+            self.set_content_length(len(response_json))
+            self.end_headers()
+            self.wfile.write(response_json)
+
         return
 
 
@@ -417,3 +491,47 @@ def handle_cgi_request(methods = None):
     # Return result
     print response_json
 
+
+def _gzip_str_to_file(raw_text, dest_file):
+    with gzip.GzipFile(filename = "", fileobj = dest_file) as gz:
+        gz.write(raw_text)
+
+
+def _gunzip_file(source_file):
+    with gzip.GzipFile(filename = "", mode = "r", fileobj = source_file) as gz:
+        return gz.read()
+
+
+class _SpooledFile(tempfile.SpooledTemporaryFile):
+    """
+    Spooled temporary file.
+
+    StringIO with fallback to temporary file if size > MAX_SIZE_IN_MEMORY.
+    """
+
+
+    def __init__(
+        self,
+        max_size = MAX_SIZE_IN_MEMORY,
+        mode = "w+b",
+        source_file = None,
+        *args, **kwargs
+    ):
+        tempfile.SpooledTemporaryFile.__init__(
+            self, max_size = max_size, mode = mode
+        )
+        if source_file:
+            for chunk in iter(lambda: source_file.read(CHUNK_SIZE), ""):
+                self.write(chunk)
+            self.seek(0)
+
+
+    def __len__(self):
+        current_pos = self.tell()
+        try:
+            self.seek(0, 2)
+            return self.tell()
+        finally:
+            self.seek(current_pos)
+
+
