@@ -3,6 +3,7 @@
 
 from bunch import Bunch
 from rpcjson import json
+from rpcerror import InternalError
 
 
 class Response(Bunch):
@@ -74,6 +75,18 @@ class Response(Bunch):
         return retdict
 
 
+    def to_string(self):
+        """
+        Returns the response as JSON-string
+        """
+
+        return json.dumps(self.to_dict())
+
+
+    # Alias
+    dumps = to_string
+
+
     @classmethod
     def from_dict(cls, response_dict):
         """
@@ -83,11 +96,26 @@ class Response(Bunch):
         error = response_dict.get("error")
         if error:
             result = None
-            error = cls.Error(
-                code = error.get("code"),
-                message = error.get("message"),
-                data = error.get("data")
-            )
+            if "code" in error:
+                # JSON-RPC Standard Error
+                error = cls.Error(
+                    code = error.get("code"),
+                    message = error.get("message"),
+                    data = error.get("data")
+                )
+            elif "fault" in error:
+                # Workaround for other library? I don't know it.
+                error = cls.Error(
+                    code = error.get("faultCode"),
+                    message = error.get("fault"),
+                    data = error.get("faultString")
+                )
+            else:
+                error = cls.Error(
+                    code = InternalError.code,
+                    message = InternalError.message,
+                    data = "\n".join(["%s: %s" % (k, v) for k, v in error])
+                )
         else:
             result = response_dict.get("result")
             error = None
@@ -100,20 +128,30 @@ class Response(Bunch):
         )
 
 
-def parse_response_json(json_string):
-    """
-    Returns a RPC-Response or a list with RPC-Responses
-    """
+    @classmethod
+    def from_string(cls, json_string):
+        """
+        Returns a Response-object or a list with Response-objects
+        """
 
-    if not json_string:
-        return
+        if not json_string:
+            return
 
-    data = json.loads(json_string)
+        data = json.loads(json_string)
 
-    if isinstance(data, list):
-        retlist = []
-        for response in data:
-            retlist.append(Response.from_dict(response))
-        return retlist
-    else:
-        return Response.from_dict(data)
+        if isinstance(data, list):
+            retlist = []
+            for response in data:
+                retlist.append(cls.from_dict(response))
+            return retlist
+        else:
+            return cls.from_dict(data)
+
+
+    # Alias
+    loads = from_string
+
+
+# Aliases
+parse_response_json = Response.from_string
+parse_response_string = Response.from_string
